#ifndef _SPR
#define _SPR


#include <hidef.h> /* for EnableInterrupts macro */
#include "derivative.h" /* include peripheral declarations */
#include "PublicTypes.h"
#include "ADC.h"
#include "AverageFilter.h"
#include "SwTimer.h"
#include "SystemConfig.h"

#define 	SPR_OUTBUFFER_DATA_SIZE					16
#define   SPR_OUTBUFFER_RESULTS_SIZE      14

#define   INH_DATA_TYPE    0    // Inhalation type
#define   EXH_DATA_TYPE    1    // Exhalation type
#define   RESULTS          2    // Results type
	
#define 	ADC_CHANNEL_SPR_PRESSURE_SIGNAL		0

#define		SPR_SAMPLING_PERIOD					10		// ms
#define		SENSOR_WARM_UP_TIME					50		// ms

#define		INHALATION_DETECTION_THRESHOLD 		7  // ADC counts

#define		IGNORED_INITIAL_SAMPLES				20		// Number of samples that will be ignored at the beginning of each measurement
														                  // this is because the moving average stabilization and the sensor warm up time

#define 	INIT_SAMPLES						200		// Initial samples to get an average of the offset value

	
extern UINT8 Spr_IsMeasurementComplete;
extern UINT8 Spr_FlowData[SPR_OUTBUFFER_DATA_SIZE];
extern UINT8 Spr_VolumeData[SPR_OUTBUFFER_DATA_SIZE];
extern UINT8 Spr_Results[SPR_OUTBUFFER_RESULTS_SIZE];
extern UINT8 Data_type;
extern UINT8 GraphBufferIndex;

void  Spr_PeriodicTask(void);
UINT8 Spr_StartMeasurement(void);
void  Spr_AbortMeasurement(void);

/** Call this function once before the main loop. */	
void Spr_Init(void);

/** Array of pointers to functions to execute Spr events */
extern const pFunc_t Spr_Events[];

/** These are the possible events that the Spr can generate */
typedef enum
{
	EVENT_SPR_NONE,
	EVENT_SPR_MEASUREMENT_COMPLETE_OK,
	EVENT_SPR_MEASUREMENT_ERROR,
	EVENT_SPR_NEW_DATA_READY
} Spr_Event_e;






#endif //_SPR