
#include <hidef.h> /* for EnableInterrupts macro */
#include "derivative.h" /* include peripheral declarations */
#include "PublicTypes.h"
#include "types.h"



#include "SwTimer.h"
#include "SystemConfig.h"
#include "SerialCommands.h"
#include "Spr.h"
#include "OPAMP.h"

#include "virtual_com.h"
#include "usb_cdc.h"


#if DEBUG_MODE==FALSE
//#define SEND_SINE_WAVE

#define CONNECTED_TO_USB	1		//PTADD_PTADD0 == 1	//Change this
#define CONNECTED_TO_LCD_BOARD	0	//IRQSC_IRQF == 1
#define GUI_TOLD_ME_TO_GO_STAND_ALONE	0//	PTADD_PTADD2	//change this


typedef union 
{
	byte Byte;
	struct 
	{
		byte ConnectedToUsb      	:1;
		byte ConnectedToLcdBoard	:1;
		byte ZigbeeEnabled			:1;
		byte BluetoothEnabled		:1;
		byte :1;
		byte :1;
		byte :1;
		byte :1;
	} Bits;
} SystemFlags;


typedef enum
{
	NO_MEASUREMENT,
	GLU_MEASUREMENT,
	BPM_MEASUREMENT,
	BPM_LEAK_TEST,
	ECG_MEASUREMENT,
	SPR_MEASUREMENT,
	HEIGHT_MEASUREMENT,
	WEIGHT_MEASUREMENT,
	TEMPERATURE_MEASUREMENT
};



UINT8 InBuffer[32];	//From the PC to the MCU
UINT8 OutBuffer[128];	//From the MCU to the PC
UINT8 OutSize;
UINT8 InSize;


/* Receive Buffer */
extern uint_8 g_curr_recv_buf[DATA_BUFF_SIZE];
/* Send Buffer */
extern uint_8 g_curr_send_buf[DATA_BUFF_SIZE];
/* Receive Data Size */
extern uint_8 g_recv_size;
/* Send Data Size */
extern uint_8 g_send_size;



UINT16 IdNumber = 0;
UINT8 MainActualState;
UINT8 ActualMeasurement;

UINT8 TimerSendDummyData;

void TimerSendDummyData_Event(void);
void EnableAFE(UINT8 measurementId);


typedef enum
{
	MAIN_STATE_DISCONNECTED,
	MAIN_STATE_CONTROLLED_BY_LCD_BOARD,
	MAIN_STATE_CONTROLLED_BY_USB
} MainStates_e;

/* States */
void StateDisconnected(void);
void StateControlledByLcdBoard(void);
void StateControlledByUsb(void);

/*****************************************************************************
 * Local Functions Prototypes
 *****************************************************************************/
static void Init_Sys(void);
static void Mcu_Init(void);
static void MCG_Init(void);

void TestApp_Init(void);
void TestApp_Task(void);


/* Spr */
void SprDiagnosticModeStartMeasurementReq(void);
void SprDiagnosticModeStopMeasurementReq(void);
void SprDiagnosticModeNewDataReadyInd(void);


/* system restart */
void SystemRestartReq(void);

/* Spr 2nd pass */
const pFunc_t Spr_Events[] = 
{
	NULL,//EVENT_SPR_NONE,
	NULL,//EVENT_SPR_MEASUREMENT_COMPLETE_OK,
	NULL,//EVENT_SPR_MEASUREMENT_ERROR,
	SprDiagnosticModeNewDataReadyInd	//EVENT_SPR_NEW_DATA_READY
};


/* Serial comm */
void SerialComm_PeriodicTask(void);
void SerialComm_SendData(void);

 
const pFunc_t MainStateMachine[] = 
{
	StateDisconnected,
	StateControlledByLcdBoard,
	StateControlledByUsb
};



const pFunc_t ExecuteCommandReq[] =
{
	NULL,				//GLU_START_MEASUREMENT,
	NULL,				//GLU_ABORT_MEASUREMENT,
	NULL,				//GLU_START_CALIBRATION,
	NULL,								//GLU_BLOOD_DETECTED,
	NULL,								//GLU_MEASUREMENT_COMPLETE_OK,
	NULL,								//GLU_CALIBRATION_COMPLETE_OK,
	
	NULL,				//BPM_START_MEASUREMENT,
	NULL,				//BPM_ABORT_MEASUREMENT,
	NULL,								//BPM_MEASUREMENT_COMPLETE_OK,
	NULL,								//BPM_MEASUREMENT_ERROR,
										  
	NULL,				//BPM_START_LEAK_TEST,
	NULL,					//BPM_ABORT_LEAK_TEST,
	NULL,								//BPM_LEAK_TEST_COMPLETE,	
	
	NULL,	//ECG_HEART_RATE_START_MEASUREMENT,
	NULL,	//ECG_HEART_RATE_ABORT_MEASUREMENT,
	NULL,								//ECG_HEART_RATE_MEASUREMENT_COMPLETE_OK,
	NULL,								//ECG_HEART_RATE_MEASUREMENT_ERROR,
	NULL,								//ECG_HEART_BEAT_OCCURRED,
	
	NULL,//ECG_DIAGNOSTIC_MODE_START_MEASUREMENT,
	NULL, //ECG_DIAGNOSTIC_MODE_STOP_MEASUREMENT,
	NULL,								 //ECG_DIAGNOSTIC_MODE_NEW_DATA_READY,	
		
	NULL,				//TMP_READ_TEMPERATURE,
	NULL,					//HGT_READ_HEIGHT,
	NULL,					//WGT_READ_WEIGHT,

	NULL,	//SprStartMeasurementReq,				//SPR_START_MEASUREMENT,
	NULL,	//SprAbortMeasurementReq,				//SPR_ABORT_MEASUREMENT,
	NULL,								//SPR_MEASUREMENT_COMPLETE_OK,
	NULL,								//SPR_MEASUREMENT_ERROR,

	SprDiagnosticModeStartMeasurementReq,				//SPR_DIAGNOSTIC_MODE_START_MEASURMENT,
	SprDiagnosticModeStopMeasurementReq,			  //SPR_DIAGNOSTIC_MODE_STOP_MEASURMENT,
	NULL,								//SPR_DIAGNOSTIC_MODE_NEW_DATA_READY,
	NULL,								//SPR_DIAGNOSTIC_MODE_MEASUREMENT_COMPLETE_OK,
	NULL,								//SPR_DIAGNOSTIC_MODE_MEASUREMENT_ERROR,
		
	NULL,				//POX_START_MEASUREMENT,
	NULL,				//POX_ABORT_MEASURMENT,
	NULL,								//POX_MEASURMENT_COMPLETE_OK,
	NULL,								//POX_MEASURMENT_ERROR,

	NULL,//POX_DIAGNOSTIC_MODE_START_MEASURMENT,
	NULL,//POX_DIAGNOSTIC_MODE_STOP_MEASURMENT,
	NULL,								//POX_DIAGNOSTIC_MODE_NEW_DATA_READY
	NULL,	//BPM_SEND_PRESSURE_VALUE_TO_PC,
	SystemRestartReq,	//SYSTEM_RESTART,
	NULL	//BPM_DATA_READY = 0xFF,
};

void TestApp_Init(void)
{
  //Enable Peripherals
  AFE_ENABLE_PIN_INIT();
  AFE_ENABLE();
  //opamp_gp_mode();
	//TRIAMP1C0_HighMode();
	//TRIAMP2C0_HighMode(); 
  SwTimer_Init();      
  Spr_Init();	
	EnableInterrupts;
    
  USB_Init();
}

void TestApp_Task (void)
{
  Spr_PeriodicTask();		
  SwTimer_PeriodicTask();
  SerialComm_PeriodicTask();
  MainStateMachine[MainActualState]();
}

static const UINT16 SinX[] = 
{
	1250, 1349, 1448, 1545, 1640, 1733, 1823, 1910, 1992, 2069, 2142, 2208, 2269, 2323, 2371, 2411, 2444, 2470, 2488, 2498, 2500, 2494, 2481, 2459, 2430, 2394, 2350, 2300, 2243, 2179, 2110, 2036, 1956, 1872, 1784, 1693, 1598, 1502, 1404, 1305, 1206, 1107, 1009, 912, 818, 726, 638, 553, 473, 398, 328, 264, 206, 155, 111, 73, 44, 21, 7, 0, 2, 11, 28, 53, 85, 124, 171, 225, 284, 350, 422, 499, 581, 667, 756, 849, 944, 1041, 1140, 1239
};

#define SIN_X_LAST_ELEMENT 	79
volatile UINT8 EcgBufferSize = 64;
volatile UINT8 Time	= 64;


void TimerSendDummyData_Event(void)
{
	if (ActualMeasurement == ECG_MEASUREMENT)
	{		
		static UINT8 SinXArrayActualElement = 0;

		//Send indication
		OutBuffer[OutSize++] = IND;
		OutBuffer[OutSize++] = ECG_DIAGNOSTIC_MODE_NEW_DATA_READY;
		OutBuffer[OutSize++] = EcgBufferSize + 2;				//data bytes
		
		OutBuffer[OutSize++] = (UINT8) (IdNumber >> 8);
		OutBuffer[OutSize++] = (UINT8) (IdNumber & 0x00FF);
		
		IdNumber++;
		
		
		while (OutSize < (EcgBufferSize + DATA_PACKET + 2))
		{
			OutBuffer[OutSize++] = SinX[SinXArrayActualElement] >> 8;		
			OutBuffer[OutSize++] = SinX[SinXArrayActualElement] & 0x00FF;
			
			if (SinXArrayActualElement == SIN_X_LAST_ELEMENT)
			{
				SinXArrayActualElement = 0;
			}
			else
			{
				SinXArrayActualElement++;
			}
		}

		(void)SerialComm_SendData();
		
		SwTimer_StartTimer(TimerSendDummyData, Time);
	}	
	
}

void StateDisconnected(void)
{
	if (CONNECTED_TO_USB)
	{
		MainActualState = MAIN_STATE_CONTROLLED_BY_USB;
	}
		else if (CONNECTED_TO_LCD_BOARD)
	{
		MainActualState = MAIN_STATE_CONTROLLED_BY_LCD_BOARD;
	}	
}


void StateControlledByLcdBoard(void)
{
	if (CONNECTED_TO_USB)
	{
		//Send LL16 to sleep
		//add code here
		MainActualState = MAIN_STATE_CONTROLLED_BY_USB;
	}
	else if (!CONNECTED_TO_LCD_BOARD)
	{
		MainActualState = MAIN_STATE_DISCONNECTED;
	}
	
	//run SPI process here for communication with LL16 
}



void StateControlledByUsb(void)
{
	if (CONNECTED_TO_LCD_BOARD)
	{
		if (!CONNECTED_TO_USB || GUI_TOLD_ME_TO_GO_STAND_ALONE)
		{
			//wake up LL16
			//add code here
			MainActualState = MAIN_STATE_CONTROLLED_BY_LCD_BOARD;
		}
	}
	else //!CONNECTED_TO_LCD_BOARD
	{
		if (!CONNECTED_TO_USB)
		{
			MainActualState = MAIN_STATE_DISCONNECTED;	
		}		
	}
		
	///*CDC_Engine();	//run CDC task
}



void SerialComm_PeriodicTask(void)
{	
 	if (MainActualState == MAIN_STATE_CONTROLLED_BY_USB)
	{
   				
		static uint_8 status = 0;///*
		//if there is data, read it
		if(g_recv_size)   //*
	  {

        InSize = g_recv_size;	    
	    
		}
	}
	else if (MainActualState == MAIN_STATE_CONTROLLED_BY_LCD_BOARD)
	{
		//check if there is new data from the LCD board
		//add SPI receive function here	
	}


	//if there is data, parse it and execute command
	if (InSize>0)   ///*   g_recv_size
	{	
		//there is new data
		if (g_curr_recv_buf[PACKET_TYPE] == REQ)
		{
			if (ExecuteCommandReq[g_curr_recv_buf[COMMAND_OPCODE]] != NULL)
			{
				//check if OPCCODE is in a valid range
				if ((g_curr_recv_buf[COMMAND_OPCODE] <= LAST_COMMAND))
				{
					ExecuteCommandReq[g_curr_recv_buf[COMMAND_OPCODE]]();
				}
			}	
		}	
		
		g_recv_size=0;
		InSize = 0;
	}
}

    





void SerialComm_SendData(void)
{
	  
	if (MainActualState == MAIN_STATE_CONTROLLED_BY_USB)
	{	
			
		if (OutSize>0)			 ///*  OutSize>0
		{					
			USB_Class_CDC_Interface_DIC_Send_Data(CONTROLLER_ID,OutBuffer,OutSize);
      OutSize = 0;
		}
	}
	else if (MainActualState == MAIN_STATE_CONTROLLED_BY_LCD_BOARD)
	{
		//add SPI send function here	

	}	
}





#define MEDICAL_SENSOR_BOARD_IIC_ADDRESS		0x03

const UINT8 CmdEnableGlu	=	0x11;	
const UINT8 CmdEnableSpr	=	0x22;	
const UINT8 CmdEnablePox	=	0x33;	
const UINT8 CmdEnableEcg	=	0x44;	
const UINT8 CmdEnableBpm	=	0x55;	
const UINT8 CmdEnableHwt	=	0x66;	
const UINT8 CmdDisableAll	=	0xFF;



void EnableAFE(UINT8 measurementId)
{
	UINT8 delay=0xFF;
	
	AFE_DISABLE();		//power off board
	
	while(delay--)	//wait a few us
	{
		;
	}
	
	IIC_DISABLE();
	vfnIICV1Init();
	AFE_ENABLE();		//power on board
	
	switch (measurementId)		//send IIC data to select device
	{
		case NO_MEASUREMENT:
			break;
			
		case GLU_MEASUREMENT:
			vfnIICV1Write(MEDICAL_SENSOR_BOARD_IIC_ADDRESS, (UINT8*)&CmdEnableGlu, 1);
			break;
			
		case BPM_MEASUREMENT:
			vfnIICV1Write(MEDICAL_SENSOR_BOARD_IIC_ADDRESS, &CmdEnableBpm, 1);
			break;
						
		case ECG_MEASUREMENT:
			vfnIICV1Write(MEDICAL_SENSOR_BOARD_IIC_ADDRESS, &CmdEnableEcg, 1);
			break;
		
		case SPR_MEASUREMENT:
			vfnIICV1Write(MEDICAL_SENSOR_BOARD_IIC_ADDRESS, &CmdEnableSpr, 1);
			break;
			
		case HEIGHT_MEASUREMENT:
			vfnIICV1Write(MEDICAL_SENSOR_BOARD_IIC_ADDRESS, &CmdEnableHwt, 1);
			break;		
		
		case WEIGHT_MEASUREMENT:
			vfnIICV1Write(MEDICAL_SENSOR_BOARD_IIC_ADDRESS, &CmdEnableHwt, 1);
			break;		
		
		case TEMPERATURE_MEASUREMENT:
			vfnIICV1Write(MEDICAL_SENSOR_BOARD_IIC_ADDRESS, &CmdEnableHwt, 1);
			break;		
	}	
}


void SystemRestartReq(void)
{
	//stop all measurements
	
	Spr_AbortMeasurement();	
}




/***************************************
		Spr
****************************************/		

void SprDiagnosticModeStartMeasurementReq(void)
{
	if (ActualMeasurement == NO_MEASUREMENT)
	{
		// Execute command and send confirm
		
		AFE_ENABLE();
		
		OutBuffer[OutSize++] = CFM;
		OutBuffer[OutSize++] = SPR_DIAGNOSTIC_MODE_START_MEASUREMENT;
		OutBuffer[OutSize++] = 1;	// Data bytes
		
		if (Spr_StartMeasurement() == TRUE)
		{		
			OutBuffer[OutSize++] = ERROR_OK;
		}
		else
		{
			OutBuffer[OutSize++] = ERROR_BUSY;
		}

		(void)SerialComm_SendData();
		
		ActualMeasurement = SPR_MEASUREMENT;
	}
}

void SprDiagnosticModeStopMeasurementReq(void)
{
	if (ActualMeasurement == SPR_MEASUREMENT)
	{	
	  // Execute command and send confirm
	  
	  Spr_AbortMeasurement();
	  
		OutBuffer[OutSize++] = CFM;
		OutBuffer[OutSize++] = SPR_ABORT_MEASUREMENT;
		OutBuffer[OutSize++] = 0;	// Data bytes

		(void)SerialComm_SendData();
		
		ActualMeasurement = NO_MEASUREMENT;
	}
}

void SprDiagnosticModeNewDataReadyInd(void)
{
UINT8 i;

	if (ActualMeasurement == SPR_MEASUREMENT)
	{ 
	  if ((Data_type == INH_DATA_TYPE) || (Data_type == EXH_DATA_TYPE))
    { 
      OutBuffer[OutSize++] = IND;
      OutBuffer[OutSize++] = SPR_DIAGNOSTIC_MODE_NEW_DATA_READY;
      OutBuffer[OutSize++] = 2*SPR_OUTBUFFER_DATA_SIZE + 1;   // Flow data + Volume data + Data type
      OutBuffer[OutSize++] = Data_type;
      
      for (i=0; i<SPR_OUTBUFFER_DATA_SIZE; i++)
	    {
	      // Copy values from flow buffer
		    OutBuffer[OutSize++] = Spr_FlowData[i];
		  }
	    
      for (i=0; i<SPR_OUTBUFFER_DATA_SIZE; i++)
	    {
	      // Copy values from volume buffer
	      OutBuffer[OutSize++] = Spr_VolumeData[i];
	    }
	    
	    (void)SerialComm_SendData();	     
    }
    
    
    else if (Data_type == RESULTS)
    {
	    OutBuffer[OutSize++] = IND;
	    OutBuffer[OutSize++] = SPR_DIAGNOSTIC_MODE_NEW_DATA_READY;
	    OutBuffer[OutSize++] = SPR_OUTBUFFER_RESULTS_SIZE + 1;  // Results + Data type
	    OutBuffer[OutSize++] = RESULTS;
	      
	    for (i=0; i<SPR_OUTBUFFER_RESULTS_SIZE; i++)
	    {
	      // Copy results
	      OutBuffer[OutSize++] = Spr_Results[i];
	    }
	      
	    (void)SerialComm_SendData();
    }
	}
}



  #endif